@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 #   Mozo Menu Editor - Simple fd.o Compliant Menu Editor
 #   Copyright (C) 2006  Travis Watkins
+#   Copyright (C) 2012-2021 MATE Developers
 #
 #   This library is free software; you can redistribute it and/or
 #   modify it under the terms of the GNU Library General Public
@@ -23,12 +24,12 @@ gi.require_version('MateMenu', '2.0')
 from gi.repository import GLib, Gio
 from gi.repository import Gtk, Gdk, GdkPixbuf
 from gi.repository import MateMenu
-import cgi
+import html
 import os
 import gettext
 import subprocess
 import shutil
-import urllib
+import urllib.parse
 try:
     from Mozo import config
     gettext.bindtextdomain(config.GETTEXT_PACKAGE,config.localedir)
@@ -53,11 +54,12 @@ class MainWindow:
     def __init__(self, datadir, version, argv):
         self.file_path = datadir
         self.version = version
-        self.editor = MenuEditor()
         Gtk.Window.set_default_icon_name('mozo')
         self.tree = Gtk.Builder()
         self.tree.set_translation_domain(GETTEXT_PACKAGE)
         self.tree.add_from_file(os.path.join(self.file_path, 'mozo.ui'))
+        self.editor = MenuEditor(undo_button=self.tree.get_object('undo_button'),
+                                 redo_button=self.tree.get_object('redo_button'))
         self.tree.connect_signals(self)
         self.setupMenuTree()
         self.setupItemTree()
@@ -71,9 +73,9 @@ class MainWindow:
         self.tree.get_object('delete_button').set_sensitive(False)
         accelgroup = Gtk.AccelGroup()
         keyval, modifier = Gtk.accelerator_parse('<Ctrl>Z')
-        accelgroup.connect(keyval, modifier, Gtk.AccelFlags.VISIBLE, self.on_mainwindow_undo)
+        accelgroup.connect(keyval, modifier, Gtk.AccelFlags.VISIBLE, self.on_undo_button_clicked)
         keyval, modifier = Gtk.accelerator_parse('<Ctrl><Shift>Z')
-        accelgroup.connect(keyval, modifier, Gtk.AccelFlags.VISIBLE, self.on_mainwindow_redo)
+        accelgroup.connect(keyval, modifier, Gtk.AccelFlags.VISIBLE, self.on_redo_button_clicked)
         keyval, modifier = Gtk.accelerator_parse('F1')
         accelgroup.connect(keyval, modifier, Gtk.AccelFlags.VISIBLE, self.on_help_button_clicked)
         self.tree.get_object('mainwindow').add_accel_group(accelgroup)
@@ -98,7 +100,7 @@ class MainWindow:
         update_items = False
         update_type = None
         item_id = None
-        if iter:
+        if iter and items[iter][3].get_desktop_file_path():
             update_items = True
             if isinstance(items[iter][3], MateMenu.TreeDirectory):
                 item_id = os.path.split(items[iter][3].get_desktop_file_path())[1]
@@ -184,7 +186,7 @@ class MainWindow:
         column.add_attribute(cell, 'markup', 1)
         column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
         menus.append_column(column)
-        menus.enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK, self.dnd_menus, Gdk.DragAction.COPY)
+        menus.enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK, self.dnd_menus, Gdk.DragAction.MOVE)
         menus.enable_model_drag_dest(self.dnd_both, Gdk.DragAction.PRIVATE)
         menus.get_selection().set_mode(Gtk.SelectionMode.BROWSE)
 
@@ -210,7 +212,7 @@ class MainWindow:
         items.append_column(column)
         self.item_store = Gtk.ListStore(bool, GdkPixbuf.Pixbuf, str, object)
         items.set_model(self.item_store)
-        items.enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK, self.dnd_items, Gdk.DragAction.COPY)
+        items.enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK, self.dnd_items, Gdk.DragAction.MOVE)
         items.enable_model_drag_dest(self.dnd_items, Gdk.DragAction.PRIVATE)
 
     def _cell_data_toggle_func(self, tree_column, renderer, model, treeiter, data=None):
@@ -238,13 +240,13 @@ class MainWindow:
     def loadMenu(self, iters, parent, depth=0):
         if depth == 0:
             icon = util.getIcon(parent)
-            iters[depth] = self.menu_store.append(None, (icon, cgi.escape(parent.get_name()), parent))
+            iters[depth] = self.menu_store.append(None, (icon, html.escape(parent.get_name()), parent))
         depth += 1
         for menu, show in self.editor.getMenus(parent):
             if show:
-                name = cgi.escape(menu.get_name())
+                name = html.escape(menu.get_name())
             else:
-                name = '<small><i>' + cgi.escape(menu.get_name()) + '</i></small>'
+                name = '<small><i>' + html.escape(menu.get_name()) + '</i></small>'
             icon = util.getIcon(menu)
             iters[depth] = self.menu_store.append(iters[depth-1], (icon, name, menu))
             self.loadMenu(iters, menu, depth)
@@ -258,14 +260,14 @@ class MainWindow:
                 name = '---'
             elif isinstance(item, MateMenu.TreeEntry):
                 if show:
-                    name = cgi.escape(item.get_app_info().get_display_name())
+                    name = html.escape(item.get_app_info().get_display_name())
                 else:
-                    name = '<small><i>' + cgi.escape(item.get_app_info().get_display_name()) + '</i></small>'
+                    name = '<small><i>' + html.escape(item.get_app_info().get_display_name()) + '</i></small>'
             else:
                 if show:
-                    name = cgi.escape(item.get_name())
+                    name = html.escape(item.get_name())
                 else:
-                    name = '<small><i>' + cgi.escape(item.get_name()) + '</i></small>'
+                    name = '<small><i>' + html.escape(item.get_name()) + '</i></small>'
             self.item_store.append((show, icon, name, item))
 
     #this is a little timeout callback to insert new items after
@@ -309,7 +311,7 @@ class MainWindow:
             parent = menus[iter][2]
         file_path = os.path.join(util.getUserDirectoryPath(), util.getUniqueFileId('mozo-made', '.directory'))
         process = subprocess.Popen(['mate-desktop-item-edit', file_path], env=os.environ)
-        GLib.timeout_add(100, self.waitForNewMenuProcess, process, parent.menu_id, file_path)
+        GLib.timeout_add(100, self.waitForNewMenuProcess, process, parent.get_menu_id(), file_path)
 
     def on_new_item_button_clicked(self, button):
         menu_tree = self.tree.get_object('menu_tree')
@@ -322,7 +324,7 @@ class MainWindow:
             parent = menus[iter][2]
         file_path = os.path.join(util.getUserItemPath(), util.getUniqueFileId('mozo-made', '.desktop'))
         process = subprocess.Popen(['mate-desktop-item-edit', file_path], env=os.environ)
-        GLib.timeout_add(100, self.waitForNewItemProcess, process, parent.menu_id, file_path)
+        GLib.timeout_add(100, self.waitForNewItemProcess, process, parent.get_menu_id(), file_path)
 
     def on_new_separator_button_clicked(self, button):
         item_tree = self.tree.get_object('item_tree')
@@ -425,7 +427,7 @@ class MainWindow:
                 item = self.drag_data
                 new_parent = menus[path][2]
                 if isinstance(item, MateMenu.TreeEntry):
-                    self.editor.copyItem(item, new_parent)
+                    self.editor.moveItem(item, new_parent)
                 elif isinstance(item, MateMenu.TreeDirectory):
                     if not self.editor.moveMenu(item, new_parent):
                         self.loadUpdates()
@@ -543,7 +545,7 @@ class MainWindow:
                 self.editor.moveSeparator(item, destination, before, after)
             context.finish(True, True, etime)
         elif str(selection.get_target()) == 'text/plain':
-            if selection.data is None:
+            if selection.get_data() is None:
                 return False
             menus, iter = self.tree.get_object('menu_tree').get_selection().get_selected()
             parent = menus[iter][2]
@@ -559,16 +561,16 @@ class MainWindow:
             else:
                 path = (len(items) - 1,)
                 after = items[path][3]
-            file_path = urllib.unquote(selection.data).strip()
+            file_path = urllib.parse.unquote(selection.get_data().decode("utf-8")).strip()
             if not file_path.startswith('file:'):
                 return
-            myfile = Gio.File(uri=file_path)
-            file_info = myfile.query_info(Gio.FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE)
+            myfile = Gio.File.new_for_uri(uri=file_path)
+            file_info = myfile.query_info(Gio.FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE, Gio.FileQueryInfoFlags.NONE)
             content_type = file_info.get_content_type()
             if content_type == 'application/x-desktop':
                 input_stream = myfile.read()
                 keyfile = GLib.KeyFile()
-                keyfile.load_from_data(input_stream.read())
+                keyfile.load_from_bytes(input_stream.read_bytes(1024*1024), GLib.KeyFileFlags.NONE)
                 self.editor.createItem(parent, before, after, KeyFile=keyfile)
             elif content_type in ('application/x-shellscript', 'application/x-executable'):
                 self.editor.createItem(parent, before, after,
@@ -593,7 +595,7 @@ class MainWindow:
         item = items[path][3]
         before = items[(path.get_indices()[0] - 1,)][3]
         if isinstance(item, MateMenu.TreeEntry):
-            self.editor.moveItem(item.get_parent(), item, before=before)
+            self.editor.moveItem(item, item.get_parent(), before=before)
         elif isinstance(item, MateMenu.TreeDirectory):
             self.editor.moveMenu(item, item.get_parent(), before=before)
         elif isinstance(item, MateMenu.TreeSeparator):
@@ -611,19 +613,19 @@ class MainWindow:
         item = items[path][3]
         after = items[path][3]
         if isinstance(item, MateMenu.TreeEntry):
-            self.editor.moveItem(item.get_parent(), item, after=after)
+            self.editor.moveItem(item, item.get_parent(), after=after)
         elif isinstance(item, MateMenu.TreeDirectory):
             self.editor.moveMenu(item, item.get_parent(), after=after)
         elif isinstance(item, MateMenu.TreeSeparator):
             self.editor.moveSeparator(item, item.get_parent(), after=after)
 
-    def on_mainwindow_undo(self, accelgroup, window, keyval, modifier):
+    def on_undo_button_clicked(self, *_args):
         self.editor.undo()
 
-    def on_mainwindow_redo(self, accelgroup, window, keyval, modifier):
+    def on_redo_button_clicked(self, *_args):
         self.editor.redo()
 
-    def on_help_button_clicked(self, *args):
+    def on_help_button_clicked(self, *_args):
         Gtk.show_uri(Gdk.Screen.get_default(), "help:mate-user-guide/menu-editor", Gtk.get_current_event_time())
 
     def on_revert_button_clicked(self, button):
